# Copyright 2014 Rackspace Inc.
#
# Author: Tim Simmons <tim.simmons@rackspace.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_config import cfg

DEFAULT_AGENT_PORT = 5358

AGENT_GROUP = cfg.OptGroup(
    name='service:agent',
    title="Configuration for the Agent Service"
)

AGENT_OPTS = [
    cfg.IntOpt('workers',
               help='Number of agent worker processes to spawn'),
    cfg.IntOpt('threads', default=1000,
               help='Number of agent greenthreads to spawn'),
    cfg.ListOpt('listen',
                default=['0.0.0.0:%d' % DEFAULT_AGENT_PORT],
                help='Agent host:port pairs to listen on'),
    cfg.IntOpt('tcp_backlog', default=100,
               help='The Agent TCP Backlog'),
    cfg.FloatOpt('tcp_recv_timeout', default=0.5,
                 help='Agent TCP Receive Timeout'),
    cfg.ListOpt('allow_notify', default=[],
                help='List of IP addresses allowed to NOTIFY The Agent'),
    cfg.ListOpt('masters', default=[],
                help='List of masters for the Agent, format ip:port'),
    cfg.StrOpt('backend_driver', default='bind9',
               help='The backend driver to use, e.g. bind9, djbdns, knot2'),
    cfg.StrOpt('transfer_source',
               help='An IP address to be used to fetch zones transferred in'),
    cfg.FloatOpt('notify_delay', default=0.0,
                 help='Delay after a NOTIFY arrives for a zone that the Agent '
                      'will pause and drop subsequent NOTIFYs for that zone'),
]


def register_opts(conf):
    conf.register_group(AGENT_GROUP)
    conf.register_opts(AGENT_OPTS, group=AGENT_GROUP)


def list_opts():
    return {
        AGENT_GROUP: AGENT_OPTS
    }



# Copyright 2015 Hewlett-Packard Development Company, L.P.
# Copyright 2022 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""add_delayed_notify_column

Revision ID: a69b45715cc1
Revises: f9f969f9d85e
Create Date: 2022-07-29 21:30:12.127816

"""
from alembic import op
import sqlalchemy as sa

from designate.storage.impl_sqlalchemy.alembic import legacy_utils

# revision identifiers, used by Alembic.
revision = 'a69b45715cc1'
down_revision = 'f9f969f9d85e'
branch_labels = None
depends_on = None

# Equivalent to legacy sqlalchemy-migrate revision
# 084_add_delayed_notify_column


def upgrade() -> None:
    # Check if the equivalent legacy migration has already run
    if not legacy_utils.is_migration_needed(84):
        return

    op.add_column('zones',
                  sa.Column('delayed_notify', sa.Boolean, default=False))
    op.create_index('delayed_notify', 'zones', ['delayed_notify'])



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Author: Federico Ceratto <federico.ceratto@hpe.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import unittest

from oslo_log import log as logging
import oslotest.base

from designate import exceptions
from designate import objects
from designate import utils

LOG = logging.getLogger(__name__)


def create_test_zone():
    return objects.Zone(
        id=utils.generate_uuid(),
        name='www.example.org.',
        email='foo@example.com',
    )


class ZoneTest(oslotest.base.BaseTestCase):
    def test_init(self):
        zone = create_test_zone()
        self.assertEqual('www.example.org.', zone.name)

    def test_masters_none(self):
        zone = objects.Zone()
        self.assertRaisesRegex(
            exceptions.RelationNotLoaded,
            'masters is not loaded on Zone',
            lambda: zone.masters
        )

    def test_masters(self):
        zone = objects.Zone(
            masters=objects.ZoneMasterList.from_list([
                {'host': '1.0.0.0', 'port': 53}
            ])
        )
        self.assertEqual(
            [{'host': '1.0.0.0', 'port': 53}], zone.masters.to_list())

    def test_masters_2(self):
        zone = objects.Zone(
            masters=objects.ZoneMasterList.from_list([
                {'host': '1.0.0.0'},
                {'host': '2.0.0.0'}
            ])
        )
        self.assertEqual(2, len(zone.masters))

    def test_get_master_by_ip(self):
        zone = objects.Zone(
            masters=objects.ZoneMasterList.from_list([
                {'host': '1.0.0.0', 'port': 53},
                {'host': '2.0.0.0', 'port': 53}
            ])
        )
        m = zone.get_master_by_ip('2.0.0.0').to_data()

        self.assertEqual('2.0.0.0:53', m)

    @unittest.expectedFailure  # bug: zone.masters is not iterable
    def test_get_master_by_ip_none(self):
        zone = objects.Zone()
        master = zone.get_master_by_ip('2.0.0.0')
        self.assertFalse(master)

    def test_validate(self):
        zone = create_test_zone()
        zone.validate()

    def test_validate_invalid_secondary(self):
        zone = objects.Zone(
            type='SECONDARY',
        )
        self.assertRaisesRegex(
            exceptions.InvalidObject,
            'Provided object does not match schema',
            zone.validate
        )

    def test_validate_primary_with_masters(self):
        masters = objects.ZoneMasterList()
        masters.append(objects.ZoneMaster.from_data("10.0.0.1:53"))
        zone = objects.Zone(
            name='example.com.',
            type='PRIMARY',
            email="foo@example.com",
            masters=masters
        )
        self.assertRaisesRegex(
            exceptions.InvalidObject,
            'Provided object does not match schema',
            zone.validate
        )

    def test_validate_primary_no_email(self):
        zone = objects.Zone(
            name='example.com.',
            type='PRIMARY',
        )
        self.assertRaisesRegex(
            exceptions.InvalidObject,
            'Provided object does not match schema',
            zone.validate
        )

    def test_validate_secondary_with_email(self):
        masters = objects.ZoneMasterList()
        masters.append(objects.ZoneMaster.from_data("10.0.0.1:53"))
        zone = objects.Zone(
            name='example.com.',
            type='SECONDARY',
            email="foo@example.com",
            masters=masters
        )
        self.assertRaisesRegex(
            exceptions.InvalidObject,
            'Provided object does not match schema',
            zone.validate
        )

    def test_validate_secondary_with_ttl(self):
        masters = objects.ZoneMasterList()
        masters.append(objects.ZoneMaster.from_data("10.0.0.1:53"))
        zone = objects.Zone(
            name='example.com.',
            type='SECONDARY',
            ttl=600,
            masters=masters
        )
        self.assertRaisesRegex(
            exceptions.InvalidObject,
            'Provided object does not match schema',
            zone.validate
        )

    def test_validate_secondary_with_masters_empty_list(self):
        masters = objects.ZoneMasterList()
        zone = objects.Zone(
            name='example.com.',
            type='SECONDARY',
            masters=masters
        )
        self.assertRaisesRegex(
            exceptions.InvalidObject,
            'Provided object does not match schema',
            zone.validate
        )

    def test_validate_secondary_with_masters_none(self):
        zone = objects.Zone(
            name='example.com.',
            type='SECONDARY',
            masters=None
        )

        self.assertRaisesRegex(
            exceptions.InvalidObject,
            'Provided object does not match schema',
            zone.validate
        )



# Copyright 2013 Hewlett-Packard Development Company, L.P.
#
# Author: Kiall Mac Innes <kiall@hpe.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import sys

from oslo_config import cfg
from oslo_log import log as logging
from oslo_reports import guru_meditation_report as gmr

from designate.api import service as api_service
import designate.conf
from designate import heartbeat_emitter
from designate import service
from designate import utils
from designate import version


CONF = designate.conf.CONF
CONF.import_opt('workers', 'designate.api', group='service:api')
cfg.CONF.import_group('keystone_authtoken', 'keystonemiddleware.auth_token')


def main():
    utils.read_config('designate', sys.argv)
    logging.setup(CONF, 'designate')
    gmr.TextGuruMeditation.setup_autorun(version)

    server = api_service.Service()
    heartbeat = heartbeat_emitter.get_heartbeat_emitter(server.service_name)
    service.serve(server, workers=CONF['service:api'].workers)
    heartbeat.start()
    service.wait()



# Copyright 2012 Managed I.T.
#
# Author: Kiall Mac Innes <kiall@managedit.ie>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import abc

from designate.plugin import DriverPlugin


class Storage(DriverPlugin, metaclass=abc.ABCMeta):

    """Base class for storage plugins"""
    __plugin_ns__ = 'designate.storage'
    __plugin_type__ = 'storage'

    @abc.abstractmethod
    def create_quota(self, context, quota):
        """
        Create a Quota.

        :param context: RPC Context.
        :param quota: Quota object with the values to be created.
        """

    @abc.abstractmethod
    def get_quota(self, context, quota_id):
        """
        Get a Quota via ID.

        :param context: RPC Context.
        :param quota_id: Quota ID to get.
        """

    @abc.abstractmethod
    def find_quotas(self, context, criterion=None, marker=None,
                    limit=None, sort_key=None, sort_dir=None):
        """
        Find Quotas

        :param context: RPC Context.
        :param criterion: Criteria to filter by.
        :param marker: Resource ID from which after the requested page will
                       start after
        :param limit: Integer limit of objects of the page size after the
                      marker
        :param sort_key: Key from which to sort after.
        :param sort_dir: Direction to sort after using sort_key.
        """

    @abc.abstractmethod
    def find_quota(self, context, criterion):
        """
        Find a single Quota.

        :param context: RPC Context.
        :param criterion: Criteria to filter by.
        """

    @abc.abstractmethod
    def update_quota(self, context, quota):
        """
        Update a Quota

        :param context: RPC Context.
        :param quota: Quota to update.
        """

    @abc.abstractmethod
    def delete_quota(self, context, quota_id):
        """
        Delete a Quota via ID.

        :param context: RPC Context.
        :param quota_id: Delete a Quota via ID
        """

    @abc.abstractmethod
    def create_tld(self, context, tld):
        """
        Create a TLD.

        :param context: RPC Context.
        :param tld: Tld object with the values to be created.
        """

    @abc.abstractmethod
    def get_tld(self, context, tld_id):
        """
        Get a TLD via ID.

        :param context: RPC Context.
        :param tld_id: TLD ID to get.
        """

    @abc.abstractmethod
    def find_tlds(self, context, criterion=None, marker=None,
                  limit=None, sort_key=None, sort_dir=None):
        """
        Find TLDs

        :param context: RPC Context.
        :param criterion: Criteria to filter by.
        :param marker: Resource ID from which after the requested page will
                       start after
        :param limit: Integer limit of objects of the page size after the
                      marker
        :param sort_key: Key from which to sort after.
        :param sort_dir: Direction to sort after using sort_key.
        """

    @abc.abstractmethod
    def find_tld(self, context, criterion):
        """
        Find a single TLD.

        :param context: RPC Context.
        :param criterion: Criteria to filter by.
        """

    @abc.abstractmethod
    def update_tld(self, context, tld):
        """
        Update a TLD

        :param context: RPC Context.
        :param tld: TLD to update.
        """

    @abc.abstractmethod
    def delete_tld(self, context, tld_id):
        """
        Delete a TLD via ID.

        :param context: RPC Context.
        :param tld_id: Delete a TLD via ID
        """

    @abc.abstractmethod
    def create_tsigkey(self, context, tsigkey):
        """
        Create a TSIG Key.

        :param context: RPC Context.
        :param tsigkey: TsigKey object with the values to be created.
        """

    @abc.abstractmethod
    def find_tsigkeys(self, context, criterion=None,
                      marker=None, limit=None, sort_key=None, sort_dir=None):
        """
        Find TSIG Keys.

        :param context: RPC Context.
        :param criterion: Criteria to filter by.
        :param marker: Resource ID from which after the requested page will
                       start after
        :param limit: Integer limit of objects of the page size after the
                      marker
        :param sort_key: Key from which to sort after.
        :param sort_dir: Direction to sort after using sort_key.
        """

    @abc.abstractmethod
    def get_tsigkey(self, context, tsigkey_id):
        """
        Get a TSIG Key via ID.

        :param context: RPC Context.
        :param tsigkey_id: Server ID to get.
        """

    @abc.abstractmethod
    def update_tsigkey(self, context, tsigkey):
        """
        Update a TSIG Key

        :param context: RPC Context.
        :param tsigkey: TSIG Keyto update.
        """

    @abc.abstractmethod
    def delete_tsigkey(self, context, tsigkey_id):
        """
        Delete a TSIG Key via ID.

        :param context: RPC Context.
        :param tsigkey_id: Delete a TSIG Key via ID
        """

    @abc.abstractmethod
    def find_tenants(self, context):
        """
        Find all Tenants.

        :param context: RPC Context.
        """

    @abc.abstractmethod
    def get_tenant(self, context, tenant_id):
        """
        Get all Tenants.

        :param context: RPC Context.
        :param tenant_id: ID of the Tenant.
        """

    @abc.abstractmethod
    def count_tenants(self, context):
        """
        Count tenants

        :param context: RPC Context.
        """

    @abc.abstractmethod
    def create_zone(self, context, zone):
        """
        Create a new Zone.

        :param context: RPC Context.
        :param zone: Zone object with the values to be created.
        """

    @abc.abstractmethod
    def get_zone(self, context, zone_id):
        """
        Get a Zone via its ID.

        :param context: RPC Context.
        :param zone_id: ID of the Zone.
        """

    @abc.abstractmethod
    def find_zones(self, context, criterion=None, marker=None,
                   limit=None, sort_key=None, sort_dir=None):
        """
        Find zones

        :param context: RPC Context.
        :param criterion: Criteria to filter by.
        :param marker: Resource ID from which after the requested page will
                       start after
        :param limit: Integer limit of objects of the page size after the
                      marker
        :param sort_key: Key from which to sort after.
        :param sort_dir: Direction to sort after using sort_key.
        """

    @abc.abstractmethod
    def find_zone(self, context, criterion):
        """
        Find a single Zone.

        :param context: RPC Context.
        :param criterion: Criteria to filter by.
        """

    @abc.abstractmethod
    def update_zone(self, context, zone):
        """
        Update a Zone

        :param context: RPC Context.
        :param zone: Zone object.
        """

    @abc.abstractmethod
    def delete_zone(self, context, zone_id):
        """
        Delete a Zone

        :param context: RPC Context.
        :param zone_id: Zone ID to delete.
        """

    @abc.abstractmethod
    def purge_zone(self, context, zone):
        """
        Purge a Zone

        :param context: RPC Context.
        :param zone: Zone to delete.
        """

    @abc.abstractmethod
    def count_zones(self, context, criterion=None):
        """
        Count zones

        :param context: RPC Context.
        :param criterion: Criteria to filter by.
        """

    @abc.abstractmethod
    def create_recordset(self, context, zone_id, recordset):
        """
        Create a recordset on a given Zone ID

        :param context: RPC Context.
        :param zone_id: Zone ID to create the recordset in.
        :param recordset: RecordSet object with the values to be created.
        """

    @abc.abstractmethod
    def get_recordset(self, context, recordset_id):
        """
        Get a recordset via ID

        :param context: RPC Context.
        :param recordset_id: RecordSet ID to get
        """

    @abc.abstractmethod
    def find_recordsets(self, context, criterion=None, marker=None, limit=None,
                        sort_key=None, sort_dir=None, force_index=False):
        """
        Find RecordSets.

        :param context: RPC Context.
        :param criterion: Criteria to filter by.
        :param marker: Resource ID from which after the requested page will
                       start after
        :param limit: Integer limit of objects of the page size after the
                      marker
        :param sort_key: Key from which to sort after.
        :param sort_dir: Direction to sort after using sort_key.
        """

    @abc.abstractmethod
    def find_recordsets_axfr(self, context, criterion=None):
        """
        Find RecordSets.

        :param context: RPC Context.
        :param criterion: Criteria to filter by.
        """

    @abc.abstractmethod
    def find_recordset(self, context, criterion):
        """
        Find a single RecordSet.

        :param context: RPC Context.
        :param criterion: Criteria to filter by.
        """

    @abc.abstractmethod
    def update_recordset(self, context, recordset):
        """
        Update a recordset

        :param context: RPC Context.
        :param recordset: RecordSet to update
        """

    @abc.abstractmethod
    def delete_recordset(self, context, recordset_id):
        """
        Delete a recordset

        :param context: RPC Context.
        :param recordset_id: RecordSet ID to delete
        """

    @abc.abstractmethod
    def count_recordsets(self, context, criterion=None):
        """
        Count recordsets

        :param context: RPC Context.
        :param criterion: Criteria to filter by.
        """

    @abc.abstractmethod
    def create_record(self, context, zone_id, recordset_id, record):
        """
        Create a record on a given Zone ID

        :param context: RPC Context.
        :param zone_id: Zone ID to create the record in.
        :param recordset_id: RecordSet ID to create the record in.
        :param record: Record object with the values to be created.
        """

    @abc.abstractmethod
    def get_record(self, context, record_id):
        """
        Get a record via ID

        :param context: RPC Context.
        :param record_id: Record ID to get
        """

    @abc.abstractmethod
    def find_records(self, context, criterion=None, marker=None,
                     limit=None, sort_key=None, sort_dir=None):
        """
        Find Records.

        :param context: RPC Context.
        :param criterion: Criteria to filter by.
        :param marker: Resource ID from which after the requested page will
                       start after
        :param limit: Integer limit of objects of the page size after the
                      marker
        :param sort_key: Key from which to sort after.
        :param sort_dir: Direction to sort after using sort_key.
        """

    @abc.abstractmethod
    def find_record(self, context, criterion):
        """
        Find a single Record.

        :param context: RPC Context.
        :param criterion: Criteria to filter by.
        """

    @abc.abstractmethod
    def update_record(self, context, record):
        """
        Update a record

        :param context: RPC Context.
        :param record: Record to update
        """

    @abc.abstractmethod
    def delete_record(self, context, record_id):
        """
        Delete a record

        :param context: RPC Context.
        :param record_id: Record ID to delete
        """

    @abc.abstractmethod
    def count_records(self, context, criterion=None):
        """
        Count records

        :param context: RPC Context.
        :param criterion: Criteria to filter by.
        """

    @abc.abstractmethod
    def create_blacklist(self, context, blacklist):
        """
        Create a Blacklist.

        :param context: RPC Context.
        :param blacklist: Blacklist object with the values to be created.
        """

    @abc.abstractmethod
    def get_blacklist(self, context, blacklist_id):
        """
        Get a Blacklist via ID.

        :param context: RPC Context.
        :param blacklist_id: Blacklist ID to get.
        """

    @abc.abstractmethod
    def find_blacklists(self, context, criterion=None, marker=None,
                        limit=None, sort_key=None, sort_dir=None):
        """
        Find Blacklists

        :param context: RPC Context.
        :param criterion: Criteria to filter by.
        :param marker: Resource ID from which after the requested page will
                       start after
        :param limit: Integer limit of objects of the page size after the
                      marker
        :param sort_key: Key from which to sort after.
        :param sort_dir: Direction to sort after using sort_key.
        """

    @abc.abstractmethod
    def find_blacklist(self, context, criterion):
        """
        Find a single Blacklist.

        :param context: RPC Context.
        :param criterion: Criteria to filter by.
        """

    @abc.abstractmethod
    def update_blacklist(self, context, blacklist):
        """
        Update a Blacklist

        :param context: RPC Context.
        :param blacklist: Blacklist to update.
        """

    @abc.abstractmethod
    def delete_blacklist(self, context, blacklist_id):
        """
        Delete a Blacklist via ID.

        :param context: RPC Context.
        :param blacklist_id: Delete a Blacklist via ID
        """

    @abc.abstractmethod
    def create_pool(self, context, pool):
        """
        Create a Pool.

        :param context: RPC Context.
        :param pool: Pool object with the values to be created.
        """

    @abc.abstractmethod
    def find_pools(self, context, criterion=None, marker=None,
                   limit=None, sort_key=None, sort_dir=None):
        """
        Find all Pools

        :param context: RPC Context.
        :param criterion: Criteria by which to filter
        :param marker: Resource ID used by paging. The next page will start
                       at the next resource after the marker
        :param limit: Integer limit of objects on the page
        :param sort_key: Key used to sort the returned list
        :param sort_dir: Directions to sort after using sort_key
        """

    @abc.abstractmethod
    def find_pool(self, context, criterion):
        """
        Find a single Pool.

        :param context: RPC Context.
        :param criterion: Criteria to filter by.
        """

    @abc.abstractmethod
    def get_pool(self, context, pool_id):
        """
        Get a Pool via the id

        :param context: RPC Context.
        :param pool_id: The ID of the pool to get
        """

    @abc.abstractmethod
    def update_pool(self, context, pool):
        """
        Update the specified pool

        :param context: RPC Context.
        :param pool: Pool to update.
        """

    @abc.abstractmethod
    def delete_pool(self, context, pool_id):
        """
        Delete the pool with the matching id

        :param context: RPC Context.
        :param pool_id: The ID of the pool to be deleted
        """

    @abc.abstractmethod
    def create_pool_attribute(self, context, pool_id, pool_attribute):
        """
        Create a PoolAttribute.

        :param context: RPC Context.
        :param pool_id: The ID of the pool to which the attribute belongs.
        :param pool_attribute: PoolAttribute object with the values created.
        """

    @abc.abstractmethod
    def find_pool_attributes(self, context, criterion=None, marker=None,
                             limit=None, sort_key=None, sort_dir=None):
        """
        Find all PoolAttributes

        :param context: RPC Context
        :param criterion: Criteria by which to filer
        :param marker: Resource ID used by paging. The next page will start
                       at the next resource after the marker
        :param limit: Integer limit of objects on the page
        :param sort_key: Key used to sort the returned list
        :param sort_dir: Directions to sort after using sort_key
        """

    @abc.abstractmethod
    def find_pool_attribute(self, context, criterion):
        """
        Find a single PoolAttribute

        :param context: RPC Context.
        :param criterion: Criteria to filter by.
        """

    @abc.abstractmethod
    def get_pool_attribute(self, context, pool_attribute_id):
        """
        Get a PoolAttribute via the ID

        :param context: RPC Context.
        :param pool_attribute_id: The ID of the PoolAttribute to get
        """

    @abc.abstractmethod
    def update_pool_attribute(self, context, pool_attribute):
        """
        Update the specified pool

        :param context: RPC Context.
        :param pool_attribute: PoolAttribute to update
        """

    @abc.abstractmethod
    def delete_pool_attribute(self, context, pool_attribute_id):
        """
        Delete the pool with the matching id

        :param context: RPC Context.
        :param pool_attribute_id: The ID of the PoolAttribute to be deleted
        """

    @abc.abstractmethod
    def create_zone_import(self, context, zone_import):
        """
        Create a Zone Import.

        :param context: RPC Context.
        :param zone_import: Zone Import object with the values to be created.
        """

    @abc.abstractmethod
    def get_zone_import(self, context, zone_import_id):
        """
        Get a Zone Import via ID.

        :param context: RPC Context.
        :param zone_import_id: Zone Import ID to get.
        """

    @abc.abstractmethod
    def find_zone_imports(self, context, criterion=None, marker=None,
                          limit=None, sort_key=None, sort_dir=None):
        """
        Find Zone Imports

        :param context: RPC Context.
        :param criterion: Criteria to filter by.
        :param marker: Resource ID from which after the requested page will
                       start after
        :param limit: Integer limit of objects of the page size after the
                      marker
        :param sort_key: Key from which to sort after.
        :param sort_dir: Direction to sort after using sort_key.
        """

    @abc.abstractmethod
    def find_zone_import(self, context, criterion):
        """
        Find a single Zone Import.

        :param context: RPC Context.
        :param criterion: Criteria to filter by.
        """

    @abc.abstractmethod
    def update_zone_import(self, context, zone_import):
        """
        Update a Zone Import

        :param context: RPC Context.
        :param zone_import: Zone Import to update.
        """

    @abc.abstractmethod
    def delete_zone_import(self, context, zone_import_id):
        """
        Delete a Zone Import via ID.

        :param context: RPC Context.
        :param zone_import_id: Delete a Zone Import via ID
        """

    @abc.abstractmethod
    def create_zone_export(self, context, zone_export):
        """
        Create a Zone Export.

        :param context: RPC Context.
        :param zone_export: Zone Export object with the values to be created.
        """

    @abc.abstractmethod
    def get_zone_export(self, context, zone_export_id):
        """
        Get a Zone Export via ID.

        :param context: RPC Context.
        :param zone_export_id: Zone Export ID to get.
        """

    @abc.abstractmethod
    def find_zone_exports(self, context, criterion=None, marker=None,
                  limit=None, sort_key=None, sort_dir=None):
        """
        Find Zone Exports

        :param context: RPC Context.
        :param criterion: Criteria to filter by.
        :param marker: Resource ID from which after the requested page will
                       start after
        :param limit: Integer limit of objects of the page size after the
                      marker
        :param sort_key: Key from which to sort after.
        :param sort_dir: Direction to sort after using sort_key.
        """

    @abc.abstractmethod
    def find_zone_export(self, context, criterion):
        """
        Find a single Zone Export.

        :param context: RPC Context.
        :param criterion: Criteria to filter by.
        """

    @abc.abstractmethod
    def update_zone_export(self, context, zone_export):
        """
        Update a Zone Export

        :param context: RPC Context.
        :param zone_export: Zone Export to update.
        """

    @abc.abstractmethod
    def delete_zone_export(self, context, zone_export_id):
        """
        Delete a Zone Export via ID.

        :param context: RPC Context.
        :param zone_export_id: Delete a Zone Export via ID
        """

    @abc.abstractmethod
    def find_service_statuses(self, context, criterion=None, marker=None,
                            limit=None, sort_key=None, sort_dir=None):
        """
        Retrieve status for services

        :param context: RPC Context.
        :param criterion: Criteria to filter by.
        :param marker: Resource ID from which after the requested page will
                       start after
        :param limit: Integer limit of objects of the page size after the
                      marker
        :param sort_key: Key from which to sort after.
        :param sort_dir: Direction to sort after using sort_key.
        """

    @abc.abstractmethod
    def find_service_status(self, context, criterion):
        """
        Find a single Service Status.

        :param context: RPC Context.
        :param criterion: Criteria to filter by.
        """

    @abc.abstractmethod
    def update_service_status(self, context, service_status):
        """
        Update the Service status for a service.

        :param context: RPC Context.
        :param service_status: Set the status for a service.
        """



# Copyright 2013 Hewlett-Packard Development Company, L.P.
#
# Author: Kiall Mac Innes <kiall@hpe.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging
import pecan

from designate.api.v2.controllers import rest
from designate.objects.adapters import DesignateAdapter
from designate import utils

LOG = logging.getLogger(__name__)


class NameServersController(rest.RestController):

    @pecan.expose(template='json:', content_type='application/json')
    @utils.validate_uuid("zone_id")
    def get_all(self, zone_id):
        """List NameServers for Zone"""
        request = pecan.request
        context = request.environ['context']

        # This is a work around to overcome the fact that pool ns_records list
        # object have 2 different representations in the v2 API

        ns_records = self.central_api.get_zone_ns_records(context, zone_id)

        LOG.info("Created %(ns_records)s", {'ns_records': ns_records})

        return {
            "nameservers": DesignateAdapter.render('API_v2', ns_records,
                                                   request=request)}



# Copyright 2014 Rackspace Hosting
# All rights reserved
#
# Author: Betsy Luzader <betsy.luzader@rackspace.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from designate.tests.test_api.test_v2 import ApiV2TestCase


class ApiV2BlacklistsTest(ApiV2TestCase):
    def setUp(self):
        super(ApiV2BlacklistsTest, self).setUp()

    def test_get_blacklists(self):
        # Set the policy file as this is an admin-only API
        self.policy({'find_blacklists': '@'})

        response = self.client.get('/blacklists/')

        # Check the headers are what we expect
        self.assertEqual(200, response.status_int)
        self.assertEqual('application/json', response.content_type)

        # Check the body structure is what we expect
        self.assertIn('blacklists', response.json)
        self.assertIn('links', response.json)
        self.assertIn('self', response.json['links'])

        # Test with 0 blacklists
        self.assertEqual(0, len(response.json['blacklists']))

        data = [self.create_blacklist(
            pattern='x-%s.org.' % i) for i in range(0, 10)]

        self._assert_paging(data, '/blacklists', key='blacklists')

    def test_get_blacklist(self):
        blacklist = self.create_blacklist(fixture=0)

        # Set the policy file as this is an admin-only API
        self.policy({'get_blacklist': '@'})

        response = self.client.get('/blacklists/%s' % blacklist['id'],
                                   headers=[('Accept', 'application/json')])

        # Verify the headers
        self.assertEqual(200, response.status_int)
        self.assertEqual('application/json', response.content_type)

        # Verify the body structure
        self.assertIn('links', response.json)
        self.assertIn('self', response.json['links'])

        # Verify the returned values
        self.assertIn('id', response.json)
        self.assertIn('created_at', response.json)
        self.assertIsNone(response.json['updated_at'])
        self.assertEqual(self.get_blacklist_fixture(0)['pattern'],
                         response.json['pattern'])

    def test_get_bkaclist_invalid_id(self):
        self._assert_invalid_uuid(self.client.get, '/blacklists/%s')

    def test_create_blacklist(self):
        self.policy({'create_blacklist': '@'})
        fixture = self.get_blacklist_fixture(0)
        response = self.client.post_json('/blacklists/', fixture)

        # Verify the headers
        self.assertEqual(201, response.status_int)
        self.assertEqual('application/json', response.content_type)

        # Verify the body structure
        self.assertIn('links', response.json)
        self.assertIn('self', response.json['links'])

        # Verify the returned values
        self.assertIn('id', response.json)
        self.assertIn('created_at', response.json)
        self.assertIsNone(response.json['updated_at'])
        self.assertEqual(fixture['pattern'],
                         response.json['pattern'])

    def test_delete_blacklist(self):
        blacklist = self.create_blacklist(fixture=0)
        self.policy({'delete_blacklist': '@'})

        self.client.delete('/blacklists/%s' % blacklist['id'], status=204)

    def test_delete_bkaclist_invalid_id(self):
        self._assert_invalid_uuid(self.client.delete, '/blacklists/%s')

    def test_update_blacklist(self):
        blacklist = self.create_blacklist(fixture=0)
        self.policy({'update_blacklist': '@'})

        # Prepare the update body
        body = {'description': 'prefix-%s' % blacklist['description']}

        response = self.client.patch_json('/blacklists/%s' %
                                          blacklist['id'], body,
                                          status=200)

        # Verify the headers
        self.assertEqual(200, response.status_int)
        self.assertEqual('application/json', response.content_type)

        # Verify the body structure
        self.assertIn('links', response.json)
        self.assertIn('self', response.json['links'])

        # Verify the returned values
        self.assertIn('id', response.json)
        self.assertIsNotNone(response.json['updated_at'])
        self.assertEqual('prefix-%s' % blacklist['description'],
                         response.json['description'])

    def test_update_bkaclist_invalid_id(self):
        self._assert_invalid_uuid(self.client.patch_json, '/blacklists/%s')

    def test_get_blacklists_filter(self):
        # Add blacklists for testing
        self.policy({'create_blacklists': '@'})
        fixtures = [
            self.get_blacklist_fixture(fixture=0),
            self.get_blacklist_fixture(fixture=1)
        ]

        for fixture in fixtures:
            response = self.client.post_json('/blacklists/', fixture)

        get_urls = [
            # Filter by Pattern
            '/blacklists?pattern=blacklisted.com.',
            '/blacklists?pattern=blacklisted*'
        ]

        correct_results = [1, 2]

        for get_url, correct_result in \
                zip(get_urls, correct_results):

            self.policy({'find_blacklists': '@'})
            response = self.client.get(get_url)

            # Check the headers are what we expect
            self.assertEqual(200, response.status_int)
            self.assertEqual('application/json', response.content_type)

            # Check that the correct number of recordsets match
            self.assertEqual(correct_result, len(response.json['blacklists']))

    def test_invalid_blacklist_filter(self):
        url = '/blacklists?description=test'
        self.policy({'find_blacklists': '@'})
        self._assert_exception('bad_request', 400, self.client.get, url)

    def test_create_invalid_denylist_pattern(self):
        self.policy({'create_blacklist': '@'})
        body = {
            'description': u'This is the description.'
        }

        url = '/blacklists/'

        # doing each pattern individually so upon error one can trace
        # back to the exact line number
        body['pattern'] = ''
        self._assert_exception(
            'invalid_object', 400, self.client.post_json, url, body)

        body['pattern'] = '#(*&^%$%$#@$'
        self._assert_exception(
            'invalid_object', 400, self.client.post_json, url, body)

        body['pattern'] = 'a' * 1000
        self._assert_exception(
            'invalid_object', 400, self.client.post_json, url, body)

    def test_update_invalid_denylist_pattern(self):
        blacklist = self.create_blacklist(fixture=0)
        self.policy({'update_blacklist': '@'})

        url = ('/blacklists/%s' % blacklist['id'])

        # doing each pattern individually so upon error one can trace
        # back to the exact line number
        body = {'pattern': ''}
        self._assert_exception(
            'invalid_object', 400, self.client.patch_json, url, body,
            status=400)

        body = {'pattern': '#(*&^%$%$#@$'}
        self._assert_exception(
            'invalid_object', 400, self.client.patch_json, url, body,
            status=400)

        body = {'pattern': 'a' * 1000}
        self._assert_exception(
            'invalid_object', 400, self.client.patch_json, url, body,
            status=400)



# Copyright (c) 2014 Rackspace Hosting
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from designate.objects import base
from designate.objects import fields


@base.DesignateRegistry.register
class Record(base.DesignateObject, base.PersistentObjectMixin,
             base.DictObjectMixin):
    def __init__(self, *args, **kwargs):
        super(Record, self).__init__(*args, **kwargs)

    fields = {
        'shard': fields.IntegerFields(nullable=True, minimum=0, maximum=4095),
        'data': fields.AnyField(nullable=True),
        'zone_id': fields.UUIDFields(nullable=True),
        'managed': fields.BooleanField(nullable=True),
        'managed_resource_type': fields.StringFields(nullable=True,
                                                     maxLength=160),
        'managed_resource_id': fields.UUIDFields(nullable=True),
        'managed_plugin_name': fields.StringFields(nullable=True,
                                                   maxLength=160),
        'managed_plugin_type': fields.StringFields(nullable=True,
                                                   maxLength=160),
        'hash': fields.StringFields(nullable=True, maxLength=32),
        'description': fields.StringFields(nullable=True,
                                           maxLength=160),
        'status': fields.EnumField(
            valid_values=['ACTIVE', 'PENDING', 'ERROR', 'DELETED'],
            nullable=True
        ),
        'tenant_id': fields.StringFields(nullable=True),
        'recordset_id': fields.UUIDFields(nullable=True),
        'managed_tenant_id': fields.StringFields(nullable=True),
        'managed_resource_region': fields.StringFields(nullable=True,
                                                       maxLength=160),
        'managed_extra': fields.StringFields(nullable=True,
                                             maxLength=160),
        'action': fields.EnumField(
            valid_values=['CREATE', 'DELETE', 'UPDATE', 'NONE'],
            nullable=True
        ),
        'serial': fields.IntegerFields(nullable=True,
                                       minimum=1, maximum=4294967295),
    }

    @classmethod
    def get_recordset_schema_changes(cls):
        # This is to allow record types to override the validation on a
        # recordset
        return {}

    STRING_KEYS = [
        'id', 'recordset_id', 'data'
    ]

    def __repr__(self):
        record = self.to_dict()
        if 'data' in record:
            record['data'] = record['data'][:35]
        return self._make_obj_str(record)


@base.DesignateRegistry.register
class RecordList(base.ListObjectMixin, base.DesignateObject):
    LIST_ITEM_TYPE = Record

    fields = {
        'objects': fields.ListOfObjectsField('Record'),
    }



# Copyright 2014 Rackspace Inc.
#
# Author: Tim Simmons <tim.simmons@rackspace.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from unittest import mock

import dns.resolver

from designate.backend.agent_backend import impl_bind9
from designate import exceptions
import designate.tests
from designate.tests.unit.agent import backends
from designate import utils


class Bind9AgentBackendTestCase(designate.tests.TestCase):
    def setUp(self):
        super(Bind9AgentBackendTestCase, self).setUp()

        self.CONF.set_override('listen', ['0.0.0.0:0'], 'service:agent')

        self.backend = impl_bind9.Bind9Backend('foo')

    def test_start_backend(self):
        self.backend.start()

    def test_stop_backend(self):
        self.backend.stop()

    @mock.patch.object(dns.resolver.Resolver, 'query')
    def test_find_zone_serial(self, mock_query):
        self.assertIsNotNone(self.backend.find_zone_serial('example.org.'))

    @mock.patch.object(dns.resolver.Resolver, 'query')
    def test_find_zone_serial_query_raises(self, mock_query):
        mock_query.side_effect = Exception()
        self.assertIsNone(self.backend.find_zone_serial('example.org.'))

    @mock.patch('designate.utils.execute')
    @mock.patch('designate.backend.agent_backend.impl_bind9.Bind9Backend'
                '._sync_zone')
    def test_create_zone(self, mock_execute, mock_sync_zone):
        zone = backends.create_dnspy_zone('example.org')
        self.backend.create_zone(zone)

    @mock.patch('designate.utils.execute')
    @mock.patch('designate.backend.agent_backend.impl_bind9.Bind9Backend'
                '._sync_zone')
    def test_update_zone(self, mock_execute, mock_sync_zone):
        zone = backends.create_dnspy_zone('example.org')
        self.backend.update_zone(zone)

    @mock.patch('designate.utils.execute')
    @mock.patch('designate.backend.agent_backend.impl_bind9.Bind9Backend'
                '._sync_zone')
    def test_delete_zone(self, mock_execute, mock_sync_zone):
        self.backend.delete_zone('example.org.')

    @mock.patch('designate.utils.execute')
    def test_execute_rndc(self, mock_execute):
        self.CONF.set_override(
            'rndc_config_file', 'config_file', 'backend:agent:bind9'
        )
        self.CONF.set_override(
            'rndc_key_file', 'key_file', 'backend:agent:bind9'
        )

        self.backend._execute_rndc(self.backend._rndc_base())

        mock_execute.assert_called_once_with(
            'rndc', '-s', '127.0.0.1', '-p', '953',
            '-c', 'config_file', '-k', 'key_file'
        )

    @mock.patch('designate.utils.execute')
    def test_execute_rndc_raises(self, mock_execute):
        mock_execute.side_effect = utils.processutils.ProcessExecutionError()

        self.assertRaises(
            exceptions.Backend,
            self.backend._execute_rndc, self.backend._rndc_base()
        )

    @mock.patch('designate.utils.execute')
    @mock.patch.object(dns.zone.Zone, 'to_file')
    def test_sync_zone(self, mock_to_file, mock_execute):
        zone = backends.create_dnspy_zone('example.org')

        self.backend._sync_zone(zone)

        mock_to_file.assert_called_once_with(
            '/var/lib/designate/zones/example.org.zone', relativize=False
        )

        mock_execute.assert_called_once_with(
            'rndc', '-s', '127.0.0.1', '-p', '953', 'reload', 'example.org'
        )

    @mock.patch('designate.utils.execute')
    @mock.patch.object(dns.zone.Zone, 'to_file')
    def test_sync_zone_with_new_zone(self, mock_to_file, mock_execute):
        zone = backends.create_dnspy_zone('example.org')

        self.backend._sync_zone(zone, new_zone_flag=True)

        mock_to_file.assert_called_once_with(
            '/var/lib/designate/zones/example.org.zone', relativize=False
        )

        mock_execute.assert_called_once_with(
            'rndc', '-s', '127.0.0.1', '-p', '953', 'addzone',
            'example.org { type master; '
            'file "/var/lib/designate/zones/example.org.zone"; };'
        )



# (c) Copyright 2016 Hewlett Packard Enterprise Development Company LP.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from unittest import mock

import fixtures

from designate import exceptions
from designate import objects
from designate import policy
from designate.scheduler.filters import attribute_filter
from designate.scheduler.filters import default_pool_filter
from designate.scheduler.filters import fallback_filter
from designate.scheduler.filters import in_doubt_default_pool_filter
from designate.scheduler.filters import pool_id_attribute_filter
from designate import tests


class SchedulerFilterTest(tests.TestCase):
    def setUp(self):
        super(SchedulerFilterTest, self).setUp()
        self.context = self.get_context()
        self.zone = objects.Zone(
            name="example.com.",
            type="PRIMARY",
            email="hostmaster@example.com"
        )

        attrs = {
            'get_pool.return_value': objects.Pool(
                id="6c346011-e581-429b-a7a2-6cdf0aba91c3")
        }

        mock_storage = mock.Mock(**attrs)
        self.test_filter = self.FILTER(storage=mock_storage)


class SchedulerDefaultPoolFilterTest(SchedulerFilterTest):
    FILTER = default_pool_filter.DefaultPoolFilter

    def test_default_operation(self):
        pools = objects.PoolList.from_list(
            [{"id": "794ccc2c-d751-44fe-b57f-8894c9f5c842"}]
        )
        pools = self.test_filter.filter(self.context, pools, self.zone)

        self.assertEqual(pools[0].id, "794ccc2c-d751-44fe-b57f-8894c9f5c842")

    def test_multiple_pools(self):
        pools = objects.PoolList.from_list(
            [
                {"id": "794ccc2c-d751-44fe-b57f-8894c9f5c842"},
                {"id": "5fabcd37-262c-4cf3-8625-7f419434b6df"}
            ]
        )
        pools = self.test_filter.filter(self.context, pools, self.zone)

        self.assertEqual(pools[0].id, "794ccc2c-d751-44fe-b57f-8894c9f5c842")

    def test_no_pools(self):
        pools = objects.PoolList()
        pools = self.test_filter.filter(self.context, pools, self.zone)

        self.assertEqual(pools[0].id, "794ccc2c-d751-44fe-b57f-8894c9f5c842")


class SchedulerFallbackFilterTest(SchedulerFilterTest):
    FILTER = fallback_filter.FallbackFilter

    def test_default_operation(self):
        pools = objects.PoolList.from_list(
            [{"id": "794ccc2c-d751-44fe-b57f-8894c9f5c842"}]
        )
        pools = self.test_filter.filter(self.context, pools, self.zone)

        self.assertEqual(pools[0].id, "794ccc2c-d751-44fe-b57f-8894c9f5c842")

    def test_multiple_pools(self):
        pools = objects.PoolList.from_list(
            [
                {"id": "6c346011-e581-429b-a7a2-6cdf0aba91c3"},
                {"id": "5fabcd37-262c-4cf3-8625-7f419434b6df"}
            ]
        )
        pools = self.test_filter.filter(self.context, pools, self.zone)

        self.assertEqual(len(pools), 2)

        for pool in pools:
            self.assertIn(
                pool.id,
                [
                    "6c346011-e581-429b-a7a2-6cdf0aba91c3",
                    "5fabcd37-262c-4cf3-8625-7f419434b6df",
                ]
            )

    def test_no_pools(self):
        pools = objects.PoolList()
        pools = self.test_filter.filter(self.context, pools, self.zone)

        self.assertEqual(pools[0].id, "794ccc2c-d751-44fe-b57f-8894c9f5c842")


class SchedulerPoolIDAttributeFilterTest(SchedulerFilterTest):
    FILTER = pool_id_attribute_filter.PoolIDAttributeFilter

    def setUp(self):
        super(SchedulerPoolIDAttributeFilterTest, self).setUp()
        self.zone = objects.Zone(
            name="example.com.",
            type="PRIMARY",
            email="hostmaster@example.com",
            attributes=objects.ZoneAttributeList.from_list(
                [
                    {
                        "key": "pool_id",
                        "value": "6c346011-e581-429b-a7a2-6cdf0aba91c3"
                    }
                ]
            )
        )

    def test_default_operation(self):
        pools = objects.PoolList.from_list(
            [{"id": "6c346011-e581-429b-a7a2-6cdf0aba91c3"}]
        )
        self.useFixture(fixtures.MockPatchObject(
            policy, 'check',
            return_value=None
        ))

        pools = self.test_filter.filter(self.context, pools, self.zone)

        self.assertEqual("6c346011-e581-429b-a7a2-6cdf0aba91c3", pools[0].id)

    def test_multiple_pools(self):
        pools = objects.PoolList.from_list(
            [
                {"id": "6c346011-e581-429b-a7a2-6cdf0aba91c3"},
                {"id": "5fabcd37-262c-4cf3-8625-7f419434b6df"}
            ]
        )

        self.useFixture(fixtures.MockPatchObject(
            policy, 'check',
            return_value=None
        ))

        pools = self.test_filter.filter(self.context, pools, self.zone)

        self.assertEqual(len(pools), 1)

        self.assertEqual("6c346011-e581-429b-a7a2-6cdf0aba91c3", pools[0].id)

    def test_no_pools(self):
        pools = objects.PoolList()

        self.useFixture(fixtures.MockPatchObject(
            policy, 'check',
            return_value=None
        ))

        pools = self.test_filter.filter(self.context, pools, self.zone)

        self.assertEqual(len(pools), 0)

    def test_policy_failure(self):
        pools = objects.PoolList.from_list(
            [{"id": "6c346011-e581-429b-a7a2-6cdf0aba91c3"}]
        )

        self.useFixture(fixtures.MockPatchObject(
            policy, 'check',
            side_effect=exceptions.Forbidden
        ))

        self.assertRaises(
            exceptions.Forbidden,
            self.test_filter.filter, self.context, pools, self.zone,
        )

        policy.check.assert_called_once_with(
            'zone_create_forced_pool',
            self.context,
            pools[0])


class SchedulerAttributeFilterTest(SchedulerFilterTest):
    FILTER = attribute_filter.AttributeFilter

    def setUp(self):
        super(SchedulerAttributeFilterTest, self).setUp()
        self.zone = objects.Zone(
            name="example.com.",
            type="PRIMARY",
            email="hostmaster@example.com",
            attributes=objects.ZoneAttributeList.from_list(
                [
                    {
                        "key": "attribute_one",
                        "value": "True"
                    },
                    {
                        "key": "attribute_two",
                        "value": "False"
                    },
                    {
                        "key": "attribute_three",
                        "value": "foo"
                    }
                ]
            )
        )

    def test_default_operation(self):
        pools = objects.PoolList.from_list(
            [
                {
                    "id": "6c346011-e581-429b-a7a2-6cdf0aba91c3",
                }
            ]
        )

        pools[0].attributes = objects.PoolAttributeList.from_list(
            [
                {
                    "key": "attribute_one",
                    "value": "True"
                },
                {
                    "key": "attribute_two",
                    "value": "False"
                },
                {
                    "key": "attribute_three",
                    "value": "foo"
                }
            ])

        pools = self.test_filter.filter(self.context, pools, self.zone)

        self.assertEqual("6c346011-e581-429b-a7a2-6cdf0aba91c3", pools[0].id)

    def test_multiple_pools_all_match(self):
        pools = objects.PoolList.from_list(
            [
                {"id": "6c346011-e581-429b-a7a2-6cdf0aba91c3"},
                {"id": "5fabcd37-262c-4cf3-8625-7f419434b6df"}
            ]

        )

        attributes = objects.PoolAttributeList.from_list(
            [
                {
                    "key": "attribute_one",
                    "value": "True"
                },
                {
                    "key": "attribute_two",
                    "value": "False"
                },
                {
                    "key": "attribute_three",
                    "value": "foo"
                }
            ])

        pools[0].attributes = attributes
        pools[1].attributes = attributes

        pools = self.test_filter.filter(self.context, pools, self.zone)

        self.assertEqual(2, len(pools))

    def test_multiple_pools_one_match(self):
        pools = objects.PoolList.from_list(
            [
                {"id": "6c346011-e581-429b-a7a2-6cdf0aba91c3"},
                {"id": "5fabcd37-262c-4cf3-8625-7f419434b6df"}
            ]

        )

        pool_0_attributes = objects.PoolAttributeList.from_list(
            [
                {
                    "key": "attribute_one",
                    "value": "True"
                },
                {
                    "key": "attribute_two",
                    "value": "False"
                },
                {
                    "key": "attribute_three",
                    "value": "foo"
                }
            ])

        pool_1_attributes = objects.PoolAttributeList.from_list(
            [
                {
                    "key": "attribute_four",
                    "value": "True"
                },
                {
                    "key": "attribute_five",
                    "value": "False"
                },
                {
                    "key": "attribute_three",
                    "value": "foo"
                }
            ])

        pools[0].attributes = pool_0_attributes
        pools[1].attributes = pool_1_attributes

        pools = self.test_filter.filter(self.context, pools, self.zone)

        self.assertEqual(1, len(pools))
        self.assertEqual("6c346011-e581-429b-a7a2-6cdf0aba91c3", pools[0].id)

    def test_multiple_pools_no_match(self):
        pools = objects.PoolList.from_list(
            [
                {"id": "6c346011-e581-429b-a7a2-6cdf0aba91c3"},
                {"id": "5fabcd37-262c-4cf3-8625-7f419434b6df"}
            ]

        )

        pool_0_attributes = objects.PoolAttributeList.from_list(
            [
                {
                    "key": "attribute_six",
                    "value": "True"
                },
                {
                    "key": "attribute_two",
                    "value": "False"
                },
                {
                    "key": "attribute_seven",
                    "value": "foo"
                }
            ])

        pool_1_attributes = objects.PoolAttributeList.from_list(
            [
                {
                    "key": "attribute_four",
                    "value": "True"
                },
                {
                    "key": "attribute_five",
                    "value": "False"
                },
                {
                    "key": "attribute_three",
                    "value": "foo"
                }
            ])

        pools[0].attributes = pool_0_attributes
        pools[1].attributes = pool_1_attributes

        pools = self.test_filter.filter(self.context, pools, self.zone)

        self.assertEqual(0, len(pools))

    def test_no_match_non_bool(self):
        pools = objects.PoolList.from_list(
            [
                {"id": "6c346011-e581-429b-a7a2-6cdf0aba91c3"},
            ]

        )

        pool_0_attributes = objects.PoolAttributeList.from_list(
            [
                {
                    "key": "attribute_one",
                    "value": "True"
                },
                {
                    "key": "attribute_two",
                    "value": "False"
                },
                {
                    "key": "attribute_three",
                    "value": "bar"
                }
            ])

        pools[0].attributes = pool_0_attributes

        pools = self.test_filter.filter(self.context, pools, self.zone)

        self.assertEqual(0, len(pools))


class SchedulerInDoubtDefaultPoolFilterTest(SchedulerFilterTest):
    FILTER = in_doubt_default_pool_filter.InDoubtDefaultPoolFilter

    def test_pools_with_default(self):
        pools = objects.PoolList.from_list(
            [
                {"id": "6c346011-e581-429b-a7a2-6cdf0aba91c3"},
                {"id": "5fabcd37-262c-4cf3-8625-7f419434b6df"}
            ]
        )
        pools = self.test_filter.filter(self.context, pools, self.zone)

        self.assertEqual(1, len(pools))
        self.assertEqual(pools[0].id, "6c346011-e581-429b-a7a2-6cdf0aba91c3")

    def test_pools_without_default(self):
        pools = objects.PoolList.from_list(
            [
                {"id": "24702e43-8a52-440f-ab74-19fc16048860"},
                {"id": "5fabcd37-262c-4cf3-8625-7f419434b6df"}
            ]
        )
        pools = self.test_filter.filter(self.context, pools, self.zone)

        self.assertEqual(2, len(pools))

    def test_no_pools(self):
        pools = objects.PoolList()
        pools = self.test_filter.filter(self.context, pools, self.zone)

        self.assertEqual(0, len(pools))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_config import cfg
from oslo_log import log as logging

from designate.notification_handler import base

LOG = logging.getLogger(__name__)


class FakeHandler(base.NotificationHandler):
    __plugin_name__ = 'fake'

    def get_exchange_topics(self):
        exchange = cfg.CONF[self.name].control_exchange
        topics = cfg.CONF[self.name].notification_topics
        return exchange, topics

    def get_event_types(self):
        return cfg.CONF[self.name].allowed_event_types

    def process_notification(self, context, event_type, payload):
        LOG.info('%s: received notification - %s',
                 self.name, event_type)



# Copyright 2010 United States Government as represented by the
#   Administrator of the National Aeronautics and Space Administration.
#   All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Session Handling for SQLAlchemy backend."""

import sqlalchemy
import threading

from oslo_config import cfg
from oslo_db.sqlalchemy import session
from oslo_log import log as logging
from oslo_utils import importutils

osprofiler_sqlalchemy = importutils.try_import('osprofiler.sqlalchemy')

LOG = logging.getLogger(__name__)

CONF = cfg.CONF
try:
    CONF.import_group("profiler", "designate.service")
except cfg.NoSuchGroupError:
    pass


_FACADES = {}
_LOCK = threading.Lock()


def add_db_tracing(cache_name):
    global _LOCK

    if not osprofiler_sqlalchemy:
        return
    if not hasattr(CONF, 'profiler'):
        return
    if not CONF.profiler.enabled or not CONF.profiler.trace_sqlalchemy:
        return
    with _LOCK:
        osprofiler_sqlalchemy.add_tracing(
            sqlalchemy,
            _FACADES[cache_name].get_engine(),
            "db"
        )


def _create_facade_lazily(cfg_group, connection=None, discriminator=None):
    connection = connection or cfg.CONF[cfg_group].connection
    cache_name = "%s:%s" % (cfg_group, discriminator)

    if cache_name not in _FACADES:
        conf = dict(cfg.CONF[cfg_group].items())
        # FIXME(stephenfin): Remove this (and ideally use of
        # LegacyEngineFacade) asap since it's not compatible with SQLAlchemy
        # 2.0
        conf['autocommit'] = True
        _FACADES[cache_name] = session.EngineFacade(
            connection,
            **conf
        )
        add_db_tracing(cache_name)

    return _FACADES[cache_name]


def get_engine(cfg_group):
    facade = _create_facade_lazily(cfg_group)
    return facade.get_engine()


def get_session(cfg_group, connection=None, discriminator=None, **kwargs):
    facade = _create_facade_lazily(cfg_group, connection, discriminator)
    return facade.get_session(**kwargs)



# Copyright 2016 Rackspace Inc.
#
# Author: Eric Larson <eric.larson@rackspace.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.mport threading
from oslo_config import cfg
from oslo_config import fixture as cfg_fixture
import oslotest.base
from unittest import mock

from designate.central import rpcapi as central_rpcapi
from designate import exceptions
from designate import objects
import designate.quota.base
from designate import rpc
from designate import storage
from designate.worker import rpcapi as worker_rpcapi
from designate.worker.tasks import base


CONF = cfg.CONF


class TestTask(oslotest.base.BaseTestCase):
    def setUp(self):
        super(TestTask, self).setUp()
        self.useFixture(cfg_fixture.Config(CONF))
        self.context = mock.Mock()
        self.task = base.Task(None)
        self.storage = self.task._storage = mock.Mock()

    def test_constructor(self):
        self.assertTrue(self.task)

    @mock.patch.object(storage, 'get_storage', mock.Mock())
    def test_quota(self):
        self.assertIsInstance(self.task.quota, designate.quota.base.Quota)

    @mock.patch.object(rpc, 'get_client', mock.Mock())
    def test_central_api(self):
        self.assertIsInstance(self.task.central_api, central_rpcapi.CentralAPI)

    @mock.patch.object(rpc, 'get_client', mock.Mock())
    def test_worker_api(self):
        self.assertIsNone(self.task._worker_api)
        self.assertIsInstance(self.task.worker_api, worker_rpcapi.WorkerAPI)
        self.assertIsNotNone(self.task._worker_api)
        self.assertIsInstance(self.task.worker_api, worker_rpcapi.WorkerAPI)

    def test_compare_threshold_at_50_percentage(self):
        CONF.set_override('threshold_percentage', 50, 'service:worker')

        self.assertFalse(self.task.compare_threshold(0, 0))
        self.assertTrue(self.task.compare_threshold(3, 3))
        self.assertFalse(self.task.compare_threshold(1, 3))
        self.assertFalse(self.task.compare_threshold(2, 5))
        self.assertTrue(self.task.compare_threshold(4, 5))

    def test_compare_threshold_at_100_percentage(self):
        CONF.set_override('threshold_percentage', 100, 'service:worker')

        self.assertFalse(self.task.compare_threshold(0, 0))
        self.assertTrue(self.task.compare_threshold(3, 3))
        self.assertFalse(self.task.compare_threshold(1, 3))
        self.assertFalse(self.task.compare_threshold(2, 5))
        self.assertFalse(self.task.compare_threshold(4, 5))

    def test_current_action_is_valid(self):
        self.storage.get_zone = mock.Mock(
            return_value=objects.Zone(action='UPDATE')
        )
        self.assertTrue(
            self.task.is_current_action_valid(
                self.context, 'UPDATE', objects.Zone(action='UPDATE'))
        )

        self.storage.get_zone = mock.Mock(
            return_value=objects.Zone(action='CREATE')
        )
        self.assertTrue(
            self.task.is_current_action_valid(
                self.context, 'CREATE', objects.Zone(action='CREATE'))
        )

        self.storage.get_zone = mock.Mock(
            return_value=objects.Zone(action='UPDATE')
        )
        self.assertTrue(
            self.task.is_current_action_valid(
                self.context, 'CREATE', objects.Zone(action='CREATE'))
        )

        self.storage.get_zone = mock.Mock(
            return_value=objects.Zone(action='DELETE')
        )
        self.assertTrue(
            self.task.is_current_action_valid(
                self.context, 'DELETE', objects.Zone(action='DELETE'))
        )

    def test_current_action_delete_always_valid(self):
        self.assertTrue(
            self.task.is_current_action_valid(
                self.context, 'DELETE', None)
        )

    def test_current_action_bad_storage_always_valid(self):
        self.storage.get_zone = mock.Mock(
            side_effect=exceptions.DesignateException()
        )
        self.assertTrue(
            self.task.is_current_action_valid(
                self.context, 'CREATE', objects.Zone(action='CREATE'))
        )

    def test_current_action_is_not_valid_none(self):
        self.storage.get_zone = mock.Mock(
            return_value=objects.Zone(action='NONE')
        )
        self.assertFalse(
            self.task.is_current_action_valid(
                self.context, 'UPDATE', objects.Zone(action='UPDATE'))
        )

    def test_current_action_is_not_valid_deleted(self):
        self.storage.get_zone = mock.Mock(
            return_value=objects.Zone(action='DELETE')
        )
        self.assertFalse(
            self.task.is_current_action_valid(
                self.context, 'UPDATE', objects.Zone(action='UPDATE'))
        )

    def test_current_action_is_not_found(self):
        self.storage.get_zone = mock.Mock(
            side_effect=exceptions.ZoneNotFound()
        )
        self.assertTrue(
            self.task.is_current_action_valid(
                self.context, 'CREATE', objects.Zone(action='CREATE'))
        )

        self.storage.get_zone = mock.Mock(
            side_effect=exceptions.ZoneNotFound()
        )
        self.assertFalse(
            self.task.is_current_action_valid(
                self.context, 'UPDATE', objects.Zone(action='UPDATE'))
        )

    def test_call(self):
        self.assertRaises(NotImplementedError, self.task)



# Copyright 2014 Rackspace Inc.
#
# Author: Tim Simmons <tim.simmons@rackspace.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_config import cfg

BIND9_GROUP = cfg.OptGroup(
    name='backend:agent:bind9',
    title="Configuration for bind9 backend"
)

BINS9_OPTS = [
    cfg.StrOpt('rndc_host', default='127.0.0.1', help='RNDC Host'),
    cfg.IntOpt('rndc_port', default=953, help='RNDC Port'),
    cfg.StrOpt('rndc_config_file',
               help='RNDC Config File'),
    cfg.StrOpt('rndc_key_file', help='RNDC Key File'),
    cfg.IntOpt('rndc_timeout', default=0, min=0, help='RNDC command timeout'),
    cfg.StrOpt('zone_file_path', default='$state_path/zones',
               help='Path where zone files are stored'),
    cfg.StrOpt('query_destination', default='127.0.0.1',
               help='Host to query when finding zones'),
]


def register_opts(conf):
    conf.register_group(BIND9_GROUP)
    conf.register_opts(BINS9_OPTS, group=BIND9_GROUP)


def list_opts():
    return {
        BIND9_GROUP: BINS9_OPTS,
    }



# Copyright 2013 Hewlett-Packard Development Company, L.P.
#
# Author: Kiall Mac Innes <kiall@hp.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import eventlet
from eventlet.green import ssl
import socket

eventlet.monkey_patch(os=False)

# Work around the eventlet issue that impacts redis using TLS.
# https://github.com/eventlet/eventlet/issues/692
ssl.timeout_exc = socket.timeout

# Monkey patch the original current_thread to use the up-to-date _active
# global variable. See https://bugs.launchpad.net/bugs/1863021 and
# https://github.com/eventlet/eventlet/issues/592
import __original_module_threading as orig_threading  # noqa
import threading  # noqa
orig_threading.current_thread.__globals__['_active'] = threading._active



# Copyright 2016 Hewlett Packard Enterprise Development Company LP
# Copyright 2022 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""change_managed_column_types

Revision ID: f9f969f9d85e
Revises: bfcfc4a07487
Create Date: 2022-07-29 21:18:35.403634

"""
from alembic import op
import sqlalchemy as sa

from designate.storage.impl_sqlalchemy.alembic import legacy_utils

# revision identifiers, used by Alembic.
revision = 'f9f969f9d85e'
down_revision = 'bfcfc4a07487'
branch_labels = None
depends_on = None

# Equivalent to legacy sqlalchemy-migrate revision
# 083_change_managed_column_types


def upgrade() -> None:
    # Check if the equivalent legacy migration has already run
    if not legacy_utils.is_migration_needed(83):
        return

    with op.batch_alter_table('records') as batch_op:
        batch_op.alter_column('managed_extra', type_=sa.String(100),
                              existing_type=sa.Unicode(100),
                              existing_nullable=True)
        batch_op.alter_column('managed_plugin_type', type_=sa.String(50),
                              existing_type=sa.Unicode(50),
                              existing_nullable=True)
        batch_op.alter_column('managed_plugin_name', type_=sa.String(50),
                              existing_type=sa.Unicode(50),
                              existing_nullable=True)
        batch_op.alter_column('managed_resource_type', type_=sa.String(50),
                              existing_type=sa.Unicode(50),
                              existing_nullable=True)
        batch_op.alter_column('managed_resource_region', type_=sa.String(100),
                              existing_type=sa.Unicode(100),
                              existing_nullable=True)
        batch_op.alter_column('managed_tenant_id', type_=sa.String(36),
                              existing_type=sa.Unicode(36),
                              existing_nullable=True)



# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
# Author: Kiall Mac Innes <kiall@hpe.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import binascii
import errno
import socket
import struct
from unittest import mock

import dns
import dns.message
from oslo_log import log as logging

from designate.tests.test_mdns import MdnsTestCase

LOG = logging.getLogger(__name__)


def hex_wire(response):
    return binascii.b2a_hex(response.to_wire())


class MdnsServiceTest(MdnsTestCase):
    # DNS packet with IQUERY opcode
    query_payload = binascii.a2b_hex(
        "271209000001000000000000076578616d706c6503636f6d0000010001"
    )
    expected_response = binascii.a2b_hex(
        b"271289050001000000000000076578616d706c6503636f6d0000010001"
    )

    # expected response is an error code REFUSED.  The other fields are
    # id 10002
    # opcode IQUERY
    # rcode REFUSED
    # flags QR RD
    # ;QUESTION
    # example.com. IN A
    # ;ANSWER
    # ;AUTHORITY
    # ;ADDITIONAL

    # Use self._print_dns_msg() to display the messages

    def setUp(self):
        super(MdnsServiceTest, self).setUp()

        self.config(listen=['0.0.0.0:0'], group='service:mdns')

        self.service = self.start_service('mdns')
        self.dns_service = self.service.dns_service
        self.addr = ['0.0.0.0', 5556]

    @staticmethod
    def _print_dns_msg(desc, wire):
        """Print DNS message for debugging"""
        q = dns.message.from_wire(wire).to_text()
        print("%s:\n%s\n" % (desc, q))

    def test_stop(self):
        # NOTE: Start is already done by the fixture in start_service()
        self.service.stop()

    @mock.patch.object(dns.message, 'make_query')
    def test_handle_empty_payload(self, query_mock):
        mock_socket = mock.Mock()
        self.dns_service._dns_handle_udp_query(mock_socket, self.addr,
                                               ' '.encode('utf-8'))
        query_mock.assert_called_once_with('unknown', dns.rdatatype.A)

    def test_handle_udp_payload(self):
        mock_socket = mock.Mock()
        self.dns_service._dns_handle_udp_query(mock_socket, self.addr,
                                               self.query_payload)
        mock_socket.sendto.assert_called_once_with(self.expected_response,
                                                   self.addr)

    def test__dns_handle_tcp_conn_fail_unpack(self):
        # will call recv() only once
        mock_socket = mock.Mock()
        mock_socket.recv.side_effect = ['X', 'boo']  # X will fail unpack

        self.dns_service._dns_handle_tcp_conn(('1.2.3.4', 42), mock_socket)
        self.assertEqual(1, mock_socket.recv.call_count)
        self.assertEqual(1, mock_socket.close.call_count)

    def test__dns_handle_tcp_conn_one_query(self):
        payload = self.query_payload
        mock_socket = mock.Mock()
        pay_len = struct.pack("!H", len(payload))
        mock_socket.recv.side_effect = [pay_len, payload, socket.timeout]

        self.dns_service._dns_handle_tcp_conn(('1.2.3.4', 42), mock_socket)

        self.assertEqual(3, mock_socket.recv.call_count)
        self.assertEqual(1, mock_socket.sendall.call_count)
        self.assertEqual(1, mock_socket.close.call_count)
        wire = mock_socket.sendall.call_args[0][0]
        expected_length_raw = wire[:2]
        (expected_length,) = struct.unpack('!H', expected_length_raw)
        self.assertEqual(len(wire), expected_length + 2)
        self.assertEqual(self.expected_response, wire[2:])

    def test__dns_handle_tcp_conn_multiple_queries(self):
        payload = self.query_payload
        mock_socket = mock.Mock()
        pay_len = struct.pack("!H", len(payload))
        # Process 5 queries, then receive a misaligned query and close the
        # connection there
        mock_socket.recv.side_effect = [
            pay_len, payload,
            pay_len, payload,
            pay_len, payload,
            pay_len, payload,
            pay_len, payload,
            'X', payload,
            pay_len, payload,
            pay_len, payload,
        ]
        self.dns_service._dns_handle_tcp_conn(('1.2.3.4', 42), mock_socket)

        self.assertEqual(11, mock_socket.recv.call_count)
        self.assertEqual(5, mock_socket.sendall.call_count)
        self.assertEqual(1, mock_socket.close.call_count)

    def test__dns_handle_tcp_conn_multiple_queries_socket_error(self):
        payload = self.query_payload
        mock_socket = mock.Mock()
        pay_len = struct.pack("!H", len(payload))
        # Process 5 queries, then receive a socket error and close the
        # connection there
        mock_socket.recv.side_effect = [
            pay_len, payload,
            pay_len, payload,
            pay_len, payload,
            pay_len, payload,
            pay_len, payload,
            socket.error(errno.EAGAIN),
            pay_len, payload,
            pay_len, payload,
        ]
        self.dns_service._dns_handle_tcp_conn(('1.2.3.4', 42), mock_socket)

        self.assertEqual(11, mock_socket.recv.call_count)
        self.assertEqual(5, mock_socket.sendall.call_count)
        self.assertEqual(1, mock_socket.close.call_count)

    def test__dns_handle_tcp_conn_multiple_queries_ignore_bad_query(self):
        payload = self.query_payload
        mock_socket = mock.Mock()
        pay_len = struct.pack("!H", len(payload))
        # Ignore a broken query and keep going as long as the query len
        # header was correct
        mock_socket.recv.side_effect = [
            pay_len, payload,
            pay_len, payload[:-5] + b'hello',
            pay_len, payload,
            pay_len, payload,
            pay_len, payload,
        ]
        self.dns_service._dns_handle_tcp_conn(('1.2.3.4', 42), mock_socket)

        self.assertEqual(11, mock_socket.recv.call_count)
        self.assertEqual(4, mock_socket.sendall.call_count)
        self.assertEqual(1, mock_socket.close.call_count)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import functools
import itertools
import threading

from oslo_log import log as logging

from designate import objects

LOG = logging.getLogger(__name__)


class ZoneLockLocal(threading.local):
    def __init__(self):
        super(ZoneLockLocal, self).__init__()
        self._held = set()

    def hold(self, name):
        self._held.add(name)

    def release(self, name):
        self._held.remove(name)

    def has_lock(self, name):
        return name in self._held


def extract_zone_id(args, kwargs):
    zone_id = None

    if 'zone_id' in kwargs:
        zone_id = kwargs['zone_id']
    elif 'zone' in kwargs:
        zone_id = kwargs['zone'].id
    elif 'recordset' in kwargs:
        zone_id = kwargs['recordset'].zone_id
    elif 'record' in kwargs:
        zone_id = kwargs['record'].zone_id

    if not zone_id:
        for arg in itertools.chain(args, kwargs.values()):
            if not isinstance(arg, objects.DesignateObject):
                continue
            if isinstance(arg, objects.Zone):
                zone_id = arg.id
                if zone_id:
                    break
            elif isinstance(arg, (objects.RecordSet,
                                  objects.Record,
                                  objects.ZoneTransferRequest,
                                  objects.ZoneTransferAccept)):
                zone_id = arg.zone_id
                if zone_id:
                    break

    if not zone_id and len(args) > 1:
        arg = args[1]
        if isinstance(arg, str):
            zone_id = arg
        elif isinstance(zone_id, objects.Zone):
            zone_id = arg.id

    return zone_id


def synchronized_zone(new_zone=False):
    """Ensures only a single operation is in progress for each zone

    A Decorator which ensures only a single operation can be happening
    on a single zone at once, within the current designate-central instance
    """
    def outer(f):
        @functools.wraps(f)
        def sync_wrapper(cls, *args, **kwargs):
            if new_zone is True:
                lock_name = 'create-new-zone'
            else:
                zone_id = extract_zone_id(args, kwargs)
                if zone_id:
                    lock_name = 'zone-%s' % zone_id
                else:
                    raise Exception('Failed to determine zone id for '
                                    'synchronized operation')

            if cls.zone_lock_local.has_lock(lock_name):
                return f(cls, *args, **kwargs)

            with cls.coordination.get_lock(lock_name):
                try:
                    cls.zone_lock_local.hold(lock_name)
                    return f(cls, *args, **kwargs)
                finally:
                    cls.zone_lock_local.release(lock_name)

        return sync_wrapper
    return outer



# Copyright 2013 Hewlett-Packard Development Company, L.P.
#
# Author: Kiall Mac Innes <kiall@managedit.ie>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import itertools

from oslo_log import log as logging
from webtest import TestApp

from designate.api import middleware
from designate.api import v2 as api_v2
from designate.tests.test_api import ApiTestCase


LOG = logging.getLogger(__name__)


INVALID_ID = [
    '2fdadfb1-cf96-4259-ac6b-bb7b6d2ff98g',
    '2fdadfb1cf964259ac6bbb7b6d2ff9GG',
    '12345'
]


class ApiV2TestCase(ApiTestCase):
    def setUp(self):
        super(ApiV2TestCase, self).setUp()

        # Create the application
        self.app = api_v2.factory({})

        # Inject the NormalizeURIMiddleware middleware
        self.app = middleware.NormalizeURIMiddleware(self.app)

        # Inject the ValidationError middleware
        self.app = middleware.APIv2ValidationErrorMiddleware(self.app)

        # Inject the FaultWrapper middleware
        self.app = middleware.FaultWrapperMiddleware(self.app)

        # Inject the TestContext middleware
        self.app = middleware.TestContextMiddleware(
            self.app, self.admin_context.project_id,
            self.admin_context.project_id)

        # Obtain a test client
        self.client = TestApp(self.app)

    def tearDown(self):
        self.app = None
        self.client = None

        super(ApiV2TestCase, self).tearDown()

    def _assert_invalid_uuid(self, method, url_format, *args, **kw):
        """
        Test that UUIDs used in the URL is valid.
        """
        count = url_format.count('%s')
        for i in itertools.product(INVALID_ID, repeat=count):
            self._assert_exception('invalid_uuid', 400, method, url_format % i)

    def _assert_exception(self, expected_type, expected_status, obj,
                          *args, **kwargs):
        """
        Checks the response that a api call with a exception contains the
        wanted data.
        """
        kwargs.setdefault('status', expected_status)

        response = obj(*args, **kwargs) if not hasattr(obj, 'json') else obj

        self.assertEqual(expected_status, response.json['code'])
        self.assertEqual(expected_type, response.json['type'])

    def _assert_invalid_paging(self, data, url, key):
        """
        Test that certain circumstances is invalid for paging in a given url.
        """
        self._assert_paging(data, url, key=key,
                            limit='invalid_limit',
                            expected_type='invalid_limit',
                            expected_status=400)

        self._assert_paging(data, url, key=key,
                            sort_dir='invalid_sort_dir',
                            expected_type='invalid_sort_dir',
                            expected_status=400)

        self._assert_paging(data, url, key=key,
                            sort_key='invalid_sort_key',
                            expected_type='invalid_sort_key',
                            expected_status=400)

        self._assert_paging(data, url, key=key,
                            marker='invalid_marker',
                            expected_type='invalid_marker',
                            expected_status=400)

    def _assert_paging(self, data, url, key=None, limit=5, sort_dir='asc',
                       sort_key='created_at', marker=None,
                       expected_type=None, expected_status=200):

        def _page(marker=None):
            params = {'limit': limit,
                      'sort_dir': sort_dir,
                      'sort_key': sort_key}

            if marker is not None:
                params['marker'] = marker

            r = self.client.get(url, params, status=expected_status)
            if expected_status != 200:
                if expected_type:
                    self._assert_exception(expected_type, expected_status, r)
                return r
            else:
                return r.json[key] if key in r.json else r.json

        response = _page(marker=marker)
        if expected_status != 200:
            if expected_type:
                self._assert_exception(expected_type, expected_status,
                                       response)

            return response

        x = 0
        length = len(data)
        for i in range(0, length):
            assert data[i]['id'] == response[x]['id']

            x += 1
            # Don't bother getting a new page if we're at the last item
            if x == len(response) and i != length - 1:
                x = 0
                response = _page(response[-1:][0]['id'])

        _page(marker=response[-1:][0]['id'])



# Copyright 2013 Hewlett-Packard Development Company, L.P.
#
# Author: Kiall Mac Innes <kiall@hpe.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging
import pecan

from designate.api.v2.controllers import common
from designate.api.v2.controllers import rest
from designate import exceptions
from designate.objects.adapters import DesignateAdapter
from designate.objects import RecordSet
from designate import utils

LOG = logging.getLogger(__name__)


class RecordSetsController(rest.RestController):
    SORT_KEYS = ['created_at', 'id', 'updated_at', 'zone_id', 'tenant_id',
                 'name', 'type', 'ttl', 'records']

    @pecan.expose(template='json:', content_type='application/json')
    @utils.validate_uuid('zone_id', 'recordset_id')
    def get_one(self, zone_id, recordset_id):
        """Get RecordSet"""
        request = pecan.request
        context = request.environ['context']

        return DesignateAdapter.render('API_v2',
                                       self.central_api.get_recordset(
                                           context, zone_id, recordset_id),
                                       request=request)

    @pecan.expose(template='json:', content_type='application/json')
    @utils.validate_uuid('zone_id')
    def get_all(self, zone_id, **params):
        """List RecordSets"""
        request = pecan.request
        context = request.environ['context']
        recordsets = common.retrieve_matched_rrsets(context, self, zone_id,
                                                    **params)

        return DesignateAdapter.render('API_v2', recordsets, request=request)

    @pecan.expose(template='json:', content_type='application/json')
    @utils.validate_uuid('zone_id')
    def post_all(self, zone_id):
        """Create RecordSet"""
        request = pecan.request
        response = pecan.response
        context = request.environ['context']

        body = request.body_dict

        recordset = DesignateAdapter.parse('API_v2', body, RecordSet())

        recordset.validate()

        # SOA recordsets cannot be created manually
        if recordset.type == 'SOA':
            raise exceptions.BadRequest(
                "Creating a SOA recordset is not allowed")

        # Create the recordset
        recordset = self.central_api.create_recordset(
            context, zone_id, recordset)

        # Prepare the response headers
        if recordset['status'] == 'PENDING':
            response.status_int = 202
        else:
            response.status_int = 201

        recordset = DesignateAdapter.render('API_v2', recordset,
                                            request=request)

        response.headers['Location'] = recordset['links']['self']

        # Prepare and return the response body
        return recordset

    @pecan.expose(template='json:', content_type='application/json')
    @utils.validate_uuid('zone_id', 'recordset_id')
    def put_one(self, zone_id, recordset_id):
        """Update RecordSet"""
        request = pecan.request
        context = request.environ['context']
        body = request.body_dict
        response = pecan.response

        # Fetch the existing recordset
        recordset = self.central_api.get_recordset(context, zone_id,
                                                   recordset_id)

        # TODO(graham): Move this further down the stack
        if recordset.managed and not context.edit_managed_records:
            raise exceptions.BadRequest('Managed records may not be updated')

        # SOA recordsets cannot be updated manually
        if recordset['type'] == 'SOA':
            raise exceptions.BadRequest(
                'Updating SOA recordsets is not allowed')

        # NS recordsets at the zone root cannot be manually updated
        if recordset['type'] == 'NS':
            zone = self.central_api.get_zone(context, zone_id)
            if recordset['name'] == zone['name']:
                raise exceptions.BadRequest(
                    'Updating a root zone NS record is not allowed')

        # Convert to APIv2 Format

        recordset = DesignateAdapter.parse('API_v2', body, recordset)

        recordset.validate()

        # Persist the resource
        recordset = self.central_api.update_recordset(context, recordset)

        if recordset['status'] == 'PENDING':
            response.status_int = 202
        else:
            response.status_int = 200

        return DesignateAdapter.render('API_v2', recordset, request=request)

    @pecan.expose(template='json:', content_type='application/json')
    @utils.validate_uuid('zone_id', 'recordset_id')
    def delete_one(self, zone_id, recordset_id):
        """Delete RecordSet"""
        request = pecan.request
        response = pecan.response
        context = request.environ['context']

        # Fetch the existing recordset
        recordset = self.central_api.get_recordset(context, zone_id,
                                                   recordset_id)
        if recordset['type'] == 'SOA':
            raise exceptions.BadRequest(
                'Deleting a SOA recordset is not allowed')

        recordset = self.central_api.delete_recordset(
            context, zone_id, recordset_id)
        response.status_int = 202

        return DesignateAdapter.render('API_v2', recordset, request=request)



# Copyright 2016 Hewlett Packard Enterprise Development Company, L.P.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_log import log as logging

from designate.objects.adapters import base

LOG = logging.getLogger(__name__)


class YAMLAdapter(base.DesignateAdapter):

    ADAPTER_FORMAT = 'YAML'

    #####################
    #  Parsing methods  #
    #####################

    @classmethod
    def parse(cls, values, output_object, *args, **kwargs):
        obj = super(YAMLAdapter, cls).parse(
            cls.ADAPTER_FORMAT, values, output_object, *args, **kwargs)
        return obj

    @classmethod
    def render_object(cls, obj, *args, **kwargs):
        # The dict we will return to be rendered to JSON / output format
        r_obj = {}
        # Loop over all fields that are supposed to be output
        for key, value in cls.MODIFICATIONS['fields'].items():
            # Get properties for this field
            field_props = cls.MODIFICATIONS['fields'][key]
            # Check if it has to be renamed
            if field_props.get('rename', False):
                new_obj = getattr(obj, field_props.get('rename'))
                # if rename is specified we need to change the key
                obj_key = field_props.get('rename')
            else:
                # if not, move on
                new_obj = getattr(obj, key, None)
                obj_key = key
            # Check if this item is a relation (another DesignateObject that
            # will need to be converted itself
            if hasattr(obj.FIELDS.get(obj_key, {}), 'objname'):
                # Get a adapter for the nested object
                # Get the class the object is and get its adapter, then set
                # the item in the dict to the output
                adapter = cls.get_object_adapter(obj.FIELDS[obj_key].objname)
                r_obj[key] = adapter.render(cls.ADAPTER_FORMAT, new_obj, *args,
                                            **kwargs)
            elif all(hasattr(obj.FIELDS.get(obj_key, {}), attr)
                     for attr in ['min', 'max']):
                r_obj[key] = int(new_obj)
            elif new_obj is not None:
                # Just attach the damn item if there is no weird edge cases
                r_obj[key] = str(new_obj)
        # Send it back
        return r_obj

    @classmethod
    def render_list(cls, list_objects, *args, **kwargs):
        r_list = []
        for obj in list_objects:
            adapter = cls.get_object_adapter(obj)
            r_list.append(
                adapter.render(cls.ADAPTER_FORMAT, obj, *args, **kwargs)
            )
        return r_list



# Copyright 2014 Rackspace Inc.
#
# Author: Tim Simmons <tim.simmons@rackspace.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
    backend.agent
    ~~~~~~~~~~~~~
    Agent backend for Pool Manager.
    Sends DNS requests to a remote agent using private OPCODEs to trigger
    creation / deletion / update of zones.

    Configured in the [service:pool_manager] section
"""

import dns
import dns.exception
import dns.flags
import dns.message
import dns.opcode
import dns.rcode
import dns.rdataclass
import dns.rdatatype
from oslo_config import cfg
from oslo_log import log as logging

from designate.backend import base
from designate.backend import private_codes
from designate.conf.agent import DEFAULT_AGENT_PORT
from designate import dnsutils
from designate import exceptions


LOG = logging.getLogger(__name__)
CONF = cfg.CONF


class AgentPoolBackend(base.Backend):
    __plugin_name__ = 'agent'
    __backend_status__ = 'untested'

    def __init__(self, target):
        super(AgentPoolBackend, self).__init__(target)
        self.host = self.options.get('host', '127.0.0.1')
        self.port = int(self.options.get('port', DEFAULT_AGENT_PORT))
        self.timeout = CONF['service:worker'].poll_timeout
        self.retry_interval = CONF['service:worker'].poll_retry_interval
        self.max_retries = CONF['service:worker'].poll_max_retries
        # FIXME: the agent retries creating zones without any interval

    def create_zone(self, context, zone):
        LOG.debug('Create Zone')
        response = self._make_and_send_dns_message(
            zone.name,
            self.timeout,
            private_codes.CC,
            private_codes.CREATE,
            private_codes.CLASSCC,
            self.host,
            self.port
        )
        if response is None:
            raise exceptions.Backend('Failed create_zone()')

    def update_zone(self, context, zone):
        LOG.debug('Update Zone')

    def delete_zone(self, context, zone):
        LOG.debug('Delete Zone')
        response = self._make_and_send_dns_message(
            zone.name,
            self.timeout,
            private_codes.CC,
            private_codes.DELETE,
            private_codes.CLASSCC,
            self.host,
            self.port
        )
        if response is None:
            raise exceptions.Backend('Failed delete_zone()')

    def _make_and_send_dns_message(self, zone_name, timeout, opcode,
                                   rdatatype, rdclass, dest_ip,
                                   dest_port):
        dns_message = self._make_dns_message(
            zone_name, opcode, rdatatype, rdclass
        )
        LOG.info(
            "Sending '%(msg)s' for '%(zone)s' to '%(server)s:%(port)d'.",
            {
                'msg': str(opcode),
                'zone': zone_name,
                'server': dest_ip,
                'port': dest_port
            }
        )
        try:
            response = dnsutils.send_dns_message(
                dns_message, dest_ip, port=dest_port, timeout=timeout
            )
            # Check that we actually got a NOERROR in the rcode and and an
            # authoritative answer
            if not (response.flags & dns.flags.AA) or dns.rcode.from_flags(
                    response.flags, response.ednsflags) != dns.rcode.NOERROR:
                LOG.warning(
                    "Failed to get expected response while trying to "
                    "send '%(msg)s' for '%(zone)s' to "
                    "'%(server)s:%(port)d'. Response message: %(resp)s",
                    {
                        'msg': str(opcode),
                        'zone': zone_name,
                        'server': dest_ip,
                        'port': dest_port,
                        'resp': str(response)
                    }
                )
                response = None
        except dns.exception.Timeout:
            LOG.warning(
                "Got Timeout while trying to send '%(msg)s' for "
                "'%(zone)s' to '%(server)s:%(port)d'. "
                "Timeout='%(timeout)d' seconds.",
                {
                    'msg': str(opcode),
                    'zone': zone_name,
                    'server': dest_ip,
                    'port': dest_port,
                    'timeout': timeout,
                }
            )
            response = None
        except dns.query.BadResponse:
            LOG.warning(
                "Got BadResponse while trying to send '%(msg)s' for "
                "'%(zone)s' to '%(server)s:%(port)d'. "
                "Timeout='%(timeout)d' seconds.",
                {
                    'msg': str(opcode),
                    'zone': zone_name,
                    'server': dest_ip,
                    'port': dest_port,
                    'timeout': timeout,
                }
            )
            response = None

        return response

    @staticmethod
    def _make_dns_message(zone_name, opcode, rdatatype, rdclass):
        dns_message = dns.message.make_query(zone_name, rdatatype,
                                             rdclass=rdclass)
        dns_message.flags = 0

        dns_message.set_opcode(opcode)
        dns_message.flags |= dns.flags.AA

        return dns_message



# Copyright 2016 Rackspace
#
# Author: Rahman Syed <rahman.syed@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oslo_log import log as logging
import oslotest.base

from designate import exceptions
from designate import objects

LOG = logging.getLogger(__name__)


class RRDataSSHTPTest(oslotest.base.BaseTestCase):
    def test_parse_sshfp(self):
        sshfp_record = objects.SSHFP()
        sshfp_record.from_string(
            '0 0 72d30d211ce8c464de2811e534de23b9be9b4dc4')

        self.assertEqual(0, sshfp_record.algorithm)
        self.assertEqual(0, sshfp_record.fp_type)
        self.assertEqual('72d30d211ce8c464de2811e534de23b9be9b4dc4',
                         sshfp_record.fingerprint)

    def test_validate_sshfp_signed_zero_alg(self):
        recordset = objects.RecordSet(
            name='www.example.org.', type='SSHFP',
            records=objects.RecordList(objects=[
                objects.Record(
                    data='-0 0 72d30d211ce8c464de2811e534de23b9be9b4dc4',
                    status='ACTIVE'),
            ])
        )
        self.assertRaisesRegex(
            exceptions.InvalidObject,
            'Provided object does not match schema',
            recordset.validate
        )

    def test_validate_sshfp_signed_zero_fptype(self):
        recordset = objects.RecordSet(
            name='www.example.org.', type='SSHFP',
            records=objects.RecordList(objects=[
                objects.Record(
                    data='0 -0 72d30d211ce8c464de2811e534de23b9be9b4dc4',
                    status='ACTIVE'),
            ])
        )
        self.assertRaisesRegex(
            exceptions.InvalidObject,
            'Provided object does not match schema',
            recordset.validate
        )



# Copyright 2016 Hewlett Packard Enterprise Development Company LP
# Copyright 2022 Red Hat
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""add_services

Revision ID: 304d41c3847a
Revises: d04819112169
Create Date: 2022-08-01 16:41:55.139558

"""
from alembic import op
import sqlalchemy as sa

from designate.sqlalchemy.types import UUID
from designate.storage.impl_sqlalchemy.alembic import legacy_utils
from designate import utils

# revision identifiers, used by Alembic.
revision = '304d41c3847a'
down_revision = 'd04819112169'
branch_labels = None
depends_on = None

SERVICE_STATES = ["UP", "DOWN", "WARNING"]

# Equivalent to legacy sqlalchemy-migrate revision 097_add_services


def upgrade() -> None:
    # Check if the equivalent legacy migration has already run
    if not legacy_utils.is_migration_needed(97):
        return

    metadata = sa.MetaData()

    op.create_table(
        'service_statuses', metadata,
        sa.Column('id', UUID, default=utils.generate_uuid, primary_key=True),
        sa.Column('created_at', sa.DateTime),
        sa.Column('updated_at', sa.DateTime),
        sa.Column('service_name', sa.String(40), nullable=False),
        sa.Column('hostname', sa.String(255), nullable=False),
        sa.Column('heartbeated_at', sa.DateTime, nullable=True),
        sa.Column('status', sa.Enum(name='service_statuses_enum',
                                    *SERVICE_STATES), nullable=False),
        sa.Column('stats', sa.Text, nullable=False),
        sa.Column('capabilities', sa.Text, nullable=False))



# Author: Federico Ceratto <federico.ceratto@hpe.com>
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

import dns
import dns.query
import dns.rdataclass
import dns.rdatatype

from oslo_config import cfg
from oslo_config import fixture as cfg_fixture
import oslotest.base

import designate.backend.agent as agent
import designate.backend.private_codes as pcodes
from designate import context
from designate import dnsutils
from designate import exceptions
from designate import objects
from designate.tests.unit import RoObject

CONF = cfg.CONF


class AgentBackendTestCase(oslotest.base.BaseTestCase):
    def setUp(self):
        super(AgentBackendTestCase, self).setUp()
        self.useFixture(cfg_fixture.Config(CONF))

        self.context = mock.Mock()
        self.admin_context = mock.Mock()
        mock.patch.object(
            context.DesignateContext, 'get_admin_context',
            return_value=self.admin_context).start()

        CONF.set_override('poll_timeout', 1, 'service:worker')
        CONF.set_override('poll_retry_interval', 4, 'service:worker')
        CONF.set_override('poll_max_retries', 5, 'service:worker')
        CONF.set_override('poll_delay', 6, 'service:worker')

        self.zone = objects.Zone(
            id='e2bed4dc-9d01-11e4-89d3-123b93f75cba',
            name='example.com.',
            email='example@example.com',
        )
        self.target = {
            'id': '4588652b-50e7-46b9-b688-a9bad40a873e',
            'type': 'agent',
            'masters': [],
            'options': [
                {'key': 'host', 'value': 2},
                {'key': 'port', 'value': 3},
            ],
        }

        self.backend = agent.AgentPoolBackend(
            objects.PoolTarget.from_dict(self.target)
        )

    def test_create_zone(self):
        self.backend._make_and_send_dns_message = mock.Mock(
            return_value=1
        )

        out = self.backend.create_zone(self.context, self.zone)

        self.backend._make_and_send_dns_message.assert_called_with(
            self.zone.name, 1, 14, pcodes.CREATE, pcodes.SUCCESS, 2, 3)
        self.assertIsNone(out)

    def test_create_zone_exception(self):
        self.backend._make_and_send_dns_message = mock.Mock(
            return_value=None
        )

        self.assertRaisesRegex(
            exceptions.Backend, 'Failed create_zone()',
            self.backend.create_zone, self.context, self.zone,
        )

        self.backend._make_and_send_dns_message.assert_called_with(
            self.zone.name, 1, 14, pcodes.CREATE, pcodes.SUCCESS, 2, 3)

    def test_update_zone(self):
        self.assertIsNone(self.backend.update_zone(self.context, self.zone))

    def test_delete_zone(self):
        self.backend._make_and_send_dns_message = mock.Mock(
            return_value=(1, 2))

        out = self.backend.delete_zone(self.context, self.zone)

        self.backend._make_and_send_dns_message.assert_called_with(
            self.zone.name, 1, 14, pcodes.DELETE, pcodes.SUCCESS, 2, 3)
        self.assertIsNone(out)

    def test_delete_zone_exception(self):
        self.backend._make_and_send_dns_message = mock.Mock(
            return_value=None
        )

        self.assertRaisesRegex(
            exceptions.Backend, 'Failed delete_zone()',
            self.backend.delete_zone, self.context, self.zone,
        )

        self.backend._make_and_send_dns_message.assert_called_with(
            self.zone.name, 1, 14, pcodes.DELETE, pcodes.SUCCESS, 2, 3)

    @mock.patch.object(dnsutils, 'send_dns_message')
    def test_make_and_send_dns_message_timeout(self, mock_send_dns_message):
        self.backend._make_dns_message = mock.Mock(return_value='')
        mock_send_dns_message.side_effect = dns.exception.Timeout()

        self.assertIsNone(
            self.backend._make_and_send_dns_message('h', 123, 1, 2, 3, 4, 5)
        )

    @mock.patch.object(dnsutils, 'send_dns_message')
    def test_make_and_send_dns_message_bad_response(self,
                                                    mock_send_dns_message):
        self.backend._make_dns_message = mock.Mock(return_value='')
        mock_send_dns_message.side_effect = dns.query.BadResponse()

        self.assertIsNone(
            self.backend._make_and_send_dns_message('h', 123, 1, 2, 3, 4, 5)
        )

    @mock.patch.object(dnsutils, 'send_dns_message')
    def test_make_and_send_dns_message_missing_AA_flags(self,
                                                        mock_send_dns_message):
        self.backend._make_dns_message = mock.Mock(return_value='')
        response = RoObject(
            rcode=mock.Mock(return_value=dns.rcode.NOERROR),
            # rcode is NOERROR but (flags & dns.flags.AA) gives 0
            flags=0,
        )
        mock_send_dns_message.return_value = response

        self.assertIsNone(
            self.backend._make_and_send_dns_message('h', 123, 1, 2, 3, 4, 5)
        )

    @mock.patch.object(dnsutils, 'send_dns_message')
    def test_make_and_send_dns_message_error_flags(self,
                                                   mock_send_dns_message):
        self.backend._make_dns_message = mock.Mock(return_value='')
        response = RoObject(
            rcode=mock.Mock(return_value=dns.rcode.NOERROR),
            # rcode is NOERROR but flags are not NOERROR
            flags=123,
            ednsflags=321
        )
        mock_send_dns_message.return_value = response

        self.assertIsNone(
            self.backend._make_and_send_dns_message('h', 123, 1, 2, 3, 4, 5)
        )

    @mock.patch.object(dnsutils, 'send_dns_message')
    def test_make_and_send_dns_message(self, mock_send_dns_message):
        self.backend._make_dns_message = mock.Mock(return_value='')
        response = RoObject(
            rcode=mock.Mock(return_value=dns.rcode.NOERROR),
            flags=agent.dns.flags.AA,
            ednsflags=321
        )
        mock_send_dns_message.return_value = response

        self.assertEqual(
            response,
            self.backend._make_and_send_dns_message('h', 123, 1, 2, 3, 4, 5)
        )
